import re
import papis.downloaders.base


class Downloader(papis.downloaders.base.Downloader):

    def __init__(self, url):
        papis.downloaders.base.Downloader.__init__(
            self, url, name="annualreviews"
        )
        self.expected_document_format = 'pdf'

    @classmethod
    def match(cls, url):
        # http://www.annualreviews.org/doi/blahblah/blahblah
        if re.match(r".*annualreviews.org.*", url):
            return Downloader(url)
        else:
            return False

    def get_doi(self):
        mdoi = re.match(r'.*annualreviews.org/doi/(.*)', self.get_url())
        if mdoi:
            doi = mdoi.group(1).replace("abs/", "").replace("full/", "")
            self.logger.debug("[doi] = %s" % doi)
            return doi
        else:
            self.logger.error("Doi not found!!")

    def get_document_url(self):
        # http://annualreviews.org/doi/pdf/10.1146/annurev-conmatphys-031214-014726
        durl = "http://annualreviews.org/doi/pdf/%s" % self.get_doi()
        self.logger.debug("[doc url] = %s" % durl)
        return durl

    def get_bibtex_url(self):
        # http://www.annualreviews.org/action/showCitFormats?doi=10.1146/annurev-conmatphys-031214-014726
        url = "http://annualreviews.org/action/downloadCitation"\
              "?format=bibtex&cookieSet=1&doi=%s" % self.get_doi()
        self.logger.debug("[bibtex url] = %s" % url)
        return url

# vim-run: python3 %



import re
import bs4
import papis.downloaders.base
import urllib.request


class Downloader(papis.downloaders.base.Downloader):

    def __init__(self, url):
        papis.downloaders.base.Downloader.__init__(self, url, name="libgen")

    @classmethod
    def match(cls, url):
        # http://libgen.io/ads.php?md5=CBA569C45B32CA3DF52E736CD8EF6340
        if re.match(r".*libgen.*md5=.*", url):
            return Downloader(url)
        else:
            return False

    def get_md5(self):
        return re.match(r'.*md5=([A-Z0-9]+).*', self.get_url()).group(1)

    def download_bibtex(self):
        url = 'http://libgen.io/ads.php?md5=%s' % self.get_md5()
        raw_data = urllib.request.urlopen(url)\
            .read()\
            .decode('utf-8')
        soup = bs4.BeautifulSoup(raw_data, "html.parser")
        textareas = soup.find_all("textarea")
        if not textareas:
            return False
        self.bibtex_data = textareas[0].text
